Type: Package
Package: cleanr
Title: Helps You to Code Cleaner
Version: 1.4.0
Authors@R: 
    person(given = "Andreas Dominik",
           family = "Cullmann",
           role = c("aut", "cre"),
           email = "fvafrcu@mailbox.org")
Description: Check your R code for some of the most common
    layout flaws.  Many tried to teach us how to write code less dreadful,
    be it implicitly as B. W. Kernighan and D. M. Ritchie (1988)
    <ISBN:0-13-110362-8> in 'The C Programming Language' did, be it
    explicitly as R.C. Martin (2008) <ISBN:0-13-235088-2> in 'Clean Code:
    A Handbook of Agile Software Craftsmanship' did.  So we should check
    our code for files too long or wide, functions with too many lines,
    too wide lines, too many arguments or too many levels of nesting.
    Note: This is not a static code analyzer like pylint or the like.
    Checkout <https://cran.r-project.org/package=lintr> instead.
License: BSD_2_clause + file LICENSE
URL: https://gitlab.com/fvafrcu/cleanr
Depends: R (>= 3.3.0)
Imports: checkmate, fritools, pkgload, rprojroot
Suggests: devtools, rasciidoc, rmarkdown, RUnit, testthat, usethis
VignetteBuilder: rasciidoc
Encoding: UTF-8
RoxygenNote: 7.2.3
Collate: 'utils.R' 'internals.R' 'checks.R' 'cleanr-package.R'
        'options.R' 'throw.R' 'wrappers.R' 'zzz.R'
NeedsCompilation: no
Packaged: 2023-06-15 21:28:59 UTC; qwer
Author: Andreas Dominik Cullmann [aut, cre]
Maintainer: Andreas Dominik Cullmann <fvafrcu@mailbox.org>
Repository: CRAN
Date/Publication: 2023-06-16 08:30:24 UTC
Built: R 4.6.0; ; 2026-04-26 12:10:38 UTC; unix
