# Generated by roxygen2: do not edit by hand

export(check_directory)
export(check_file)
export(check_file_layout)
export(check_file_length)
export(check_file_width)
export(check_function_layout)
export(check_functions_in_file)
export(check_line_width)
export(check_nesting_depth)
export(check_num_arguments)
export(check_num_lines)
export(check_num_lines_of_code)
export(check_package)
export(check_return)
export(gco)
export(get_cleanr_options)
export(is_not_false)
export(load_internal_functions)
export(set_cleanr_options)
